"""Edge Transport Package.

This package provides edge transport abstraction for vision agents.
"""

from vision_agents.core.edge.call import Call
from vision_agents.core.edge.edge_transport import EdgeTransport

__all__ = ["Call", "EdgeTransport"]
